#include "uart.h"

#include <avr/io.h>
#include <util/delay.h>


//-----------------------------------------------------------------------------
//  Initialize USART0
//-----------------------------------------------------------------------------

void init_uart()
{
	//Uart init
	UBRRH = 0;
	UBRRL = UBRR;
	
	/* Enable receiver and transmitter */
	UCSRB = (1<<RXEN)|(1<<TXEN);
	UCSRC = (1<<URSEL)|(0 << UPM0)|(0 << UPM1)|(1 << UCSZ1)|(1 << UCSZ0);

	SREG |= _BV(SREG_I);
}

//-----------------------------------------------------------------------------
//  Set receive interrupt enable
//-----------------------------------------------------------------------------
void setRXCIE_USART0()
{
    DDRD=0X0F; 
}


//-----------------------------------------------------------------------------
//  Transmit 1 byte data
//-----------------------------------------------------------------------------
void uart_putc(unsigned char c)
{
    while (!(UCSRA & _BV(UDRE)));  	// Wait until the transmit buffer is empty
  	UDR = c; 			// As soon as the buffer is empty, put the data in UDR
}

//-----------------------------------------------------------------------------
//  Transmit string data
//-----------------------------------------------------------------------------
void uart_puts(char *str)
{
    while (*str) {
        uart_putc(*str);
        str++;
    }
}

//-----------------------------------------------------------------------------
//  Receive 1 byte data
//-----------------------------------------------------------------------------
unsigned char uart_getc(void)
{
	unsigned char c;
	while ((UCSRA & 0x80) == 0x00); // Wait for till the data arrives data
	  	c=UDR;			// Read the data from UDR, as soon as the data arrives
	return c;
}

//-----------------------------------------------------------------------------
//  Receive string with maxLen size
//-----------------------------------------------------------------------------
void uart_gets(unsigned char* buffer, unsigned int maxLen )
{
  unsigned char nextChar;
  unsigned int stringLen = 0;
 
  nextChar = uart_getc();         //Wait until the next char is arived
                                  // Put the chars in to Buffer array
  while( stringLen < maxLen - 1 ) {
    *buffer++ = nextChar;
    stringLen++;
    nextChar = uart_getc();
  }
                                  // Put '\0' at the end of the string
                                  // tog get a standard c-string
  *buffer = '\0';
}

//-----------------------------------------------------------------------------
//  Flush uart
//-----------------------------------------------------------------------------
void uart_flush( void )
{
	unsigned char dummy;
	while ( UCSRA & (1<<RXC) )
	{
		dummy = UDR;
	}
}
